@@ -10,10 +10,13 @@
 """
 
 import sys
+from os import path
 
 __revision__ = '$Revision$'
-__version__ = '0.5.2'
-__released__ = '0.5.2'
+__version__ = '0.6.2'
+__released__ = '0.6.2'
+
+package_dir = path.abspath(path.dirname(__file__))
 
 
 def main(argv=sys.argv):
@@ -28,17 +31,23 @@ def main(argv=sys.argv):
         errstr = str(err)
         if errstr.lower().startswith('no module named'):
             whichmod = errstr[16:]
+            hint = ''
             if whichmod.startswith('docutils'):
                 whichmod = 'Docutils library'
             elif whichmod.startswith('jinja'):
                 whichmod = 'Jinja library'
             elif whichmod == 'roman':
                 whichmod = 'roman module (which is distributed with Docutils)'
+                hint = ('This can happen if you upgraded docutils using\n'
+                        'easy_install without uninstalling the old version'
+                        'first.')
             else:
                 whichmod += ' module'
             print >>sys.stderr, \
                   'Error: The %s cannot be found. Did you install Sphinx '\
                   'and its dependencies correctly?' % whichmod
+            if hint:
+                print >> sys.stderr, hint
             return 1
         raise
     return cmdline.main(argv)
