@@ -5,34 +5,50 @@
 
     Format colored console output.
 
-    :copyright: 2007-2008 by Georg Brandl.
-    :license: BSD.
+    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :license: BSD, see LICENSE for details.
 """
 
+import os
+import sys
+
 codes = {}
 
 def get_terminal_width():
     """Borrowed from the py lib."""
     try:
-        import os, termios, fcntl, struct
-        call = fcntl.ioctl(0, termios.TIOCGWINSZ, "\000"*8)
-        height, width = struct.unpack("hhhh", call)[:2]
+        import termios, fcntl, struct
+        call = fcntl.ioctl(0, termios.TIOCGWINSZ,
+                           struct.pack('hhhh', 0, 0, 0, 0))
+        height, width = struct.unpack('hhhh', call)[:2]
         terminal_width = width
     except (SystemExit, KeyboardInterrupt):
         raise
     except:
         # FALLBACK
-        terminal_width = int(os.environ.get('COLUMNS', 80))-1
+        terminal_width = int(os.environ.get('COLUMNS', 80)) - 1
     return terminal_width
 
 _tw = get_terminal_width()
 
-def print_and_backspace(text, func):
+def term_width_line(text):
     if not codes:
         # if no coloring, don't output fancy backspaces
-        func(text)
+        return text + '\n'
     else:
-        func(text.ljust(_tw) + _tw * "\b")
+        return text.ljust(_tw) + '\r'
+
+def color_terminal():
+    if not hasattr(sys.stdout, 'isatty'):
+        return False
+    if not sys.stdout.isatty():
+        return False
+    if 'COLORTERM' in os.environ:
+        return True
+    term = os.environ.get('TERM', 'dumb').lower()
+    if term in ('xterm', 'linux') or 'color' in term:
+        return True
+    return False
 
 
 def nocolor():
