@@ -5,8 +5,8 @@
 
     Test the sphinx.quickstart module.
 
-    :copyright: 2008 by Georg Brandl.
-    :license: BSD.
+    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :license: BSD, see LICENSE for details.
 """
 
 import sys
@@ -38,6 +38,7 @@ def mock_raw_input(answers, needanswer=False):
 
 def teardown_module():
     qs.raw_input = __builtin__.raw_input
+    qs.TERM_ENCODING = getattr(sys.stdin, 'encoding', None)
     coloron()
 
 
@@ -64,11 +65,12 @@ def test_do_prompt():
     qs.do_prompt(d, 'k2', 'Q2')
     assert d['k2'] == 'v2'
     qs.do_prompt(d, 'k4', 'Q4', validator=qs.boolean)
-    assert d['k4'] == 'yes'
+    assert d['k4'] is True
     qs.do_prompt(d, 'k5', 'Q5', validator=qs.boolean)
-    assert d['k5'] == 'no'
+    assert d['k5'] is False
     raises(AssertionError, qs.do_prompt, d, 'k6', 'Q6', validator=qs.boolean)
 
+
 @with_tempdir
 def test_quickstart_defaults(tempdir):
     answers = {
@@ -85,40 +87,50 @@ def test_quickstart_defaults(tempdir):
     ns = {}
     execfile(conffile, ns)
     assert ns['extensions'] == []
-    assert ns['templates_path'] == ['.templates']
+    assert ns['templates_path'] == ['_templates']
     assert ns['source_suffix'] == '.rst'
     assert ns['master_doc'] == 'index'
     assert ns['project'] == 'Sphinx Test'
     assert ns['copyright'] == '%s, Georg Brandl' % time.strftime('%Y')
     assert ns['version'] == '0.1'
     assert ns['release'] == '0.1'
-    assert ns['html_static_path'] == ['.static']
+    assert ns['html_static_path'] == ['_static']
     assert ns['latex_documents'] == [
         ('index', 'SphinxTest.tex', 'Sphinx Test Documentation',
          'Georg Brandl', 'manual')]
 
-    assert (tempdir / '.static').isdir()
-    assert (tempdir / '.templates').isdir()
+    assert (tempdir / '_static').isdir()
+    assert (tempdir / '_templates').isdir()
     assert (tempdir / 'index.rst').isfile()
     assert (tempdir / 'Makefile').isfile()
+    assert (tempdir / 'make.bat').isfile()
+
 
 @with_tempdir
 def test_quickstart_all_answers(tempdir):
     answers = {
         'Root path': tempdir,
         'Separate source and build': 'y',
-        'Name prefix for templates': '_',
-        'Project name': 'Sphinx Test',
-        'Author name': 'Georg Brandl',
-        'Project version': '0.1',
-        'Project release': '0.1.1',
+        'Name prefix for templates': '.',
+        'Project name': 'STASI\xe2\x84\xa2',
+        'Author name': 'Wolfgang Sch\xc3\xa4uble & G\'Beckstein',
+        'Project version': '2.0',
+        'Project release': '2.0.1',
         'Source file suffix': '.txt',
         'Name of your master document': 'contents',
         'autodoc': 'y',
         'doctest': 'yes',
+        'intersphinx': 'no',
+        'todo': 'n',
+        'coverage': 'no',
+        'pngmath': 'N',
+        'jsmath': 'no',
+        'ifconfig': 'no',
         'Create Makefile': 'no',
+        'Create Windows command file': 'no',
     }
     qs.raw_input = mock_raw_input(answers, needanswer=True)
+    qs.TERM_ENCODING = 'utf-8'
     qs.inner_main([])
 
     conffile = tempdir / 'source' / 'conf.py'
@@ -126,19 +138,20 @@ def test_quickstart_all_answers(tempdir):
     ns = {}
     execfile(conffile, ns)
     assert ns['extensions'] == ['sphinx.ext.autodoc', 'sphinx.ext.doctest']
-    assert ns['templates_path'] == ['_templates']
+    assert ns['templates_path'] == ['.templates']
     assert ns['source_suffix'] == '.txt'
     assert ns['master_doc'] == 'contents'
-    assert ns['project'] == 'Sphinx Test'
-    assert ns['copyright'] == '%s, Georg Brandl' % time.strftime('%Y')
-    assert ns['version'] == '0.1'
-    assert ns['release'] == '0.1.1'
-    assert ns['html_static_path'] == ['_static']
+    assert ns['project'] == u'STASI™'
+    assert ns['copyright'] == u'%s, Wolfgang Schäuble & G\'Beckstein' % \
+           time.strftime('%Y')
+    assert ns['version'] == '2.0'
+    assert ns['release'] == '2.0.1'
+    assert ns['html_static_path'] == ['.static']
     assert ns['latex_documents'] == [
-        ('contents', 'SphinxTest.tex', 'Sphinx Test Documentation',
-         'Georg Brandl', 'manual')]
+        ('contents', 'STASI.tex', u'STASI™ Documentation',
+         u'Wolfgang Schäuble \\& G\'Beckstein', 'manual')]
 
     assert (tempdir / 'build').isdir()
-    assert (tempdir / 'source' / '_static').isdir()
-    assert (tempdir / 'source' / '_templates').isdir()
+    assert (tempdir / 'source' / '.static').isdir()
+    assert (tempdir / 'source' / '.templates').isdir()
     assert (tempdir / 'source' / 'contents.txt').isfile()
